"""
 cfgform - show configuration dialog form which is build from python dict

 (this is for s60/python platform only)
 
 usage:
  - construct python dictionary, using simple rules defined below
  - call cfgform.showconfig with dict parameter
  - the call returns true if user saved, false otherwise
  
 see the test() below for example

 how to use with files:
 cfg = eval(open("cfg.cfg").read())
 if (showconfig(cfg)): # write changes
   open("cfg.cfg", "w").write(repr(cfg))

 please note that using eval() is possible dangerous.

 
 dict rules: 
 - dict keys must be strings (no unicode)

 - supported dict values are:
  - unicode strings
  - numbers
  - lists of unicode strings ([<list of unicode strings>, index])

 caveats:
  - if you use stuff like "%s" % cfg['key'], the resulting string will
    be unicode
  - if you use list of unicode strings, you must access the value by
    accessing the second item (the index)

"""
import appuifw


def showconfig(cfg):        
    fields = []
    for k, v in cfg.items():
        if isinstance(v, int) or isinstance(v, long):
            tname = 'number'
            v = int(v)
        elif isinstance(v, list) or isinstance(v, tuple):
            for item in v[0]:
                if not isinstance(item, unicode):
                    raise Exception("list can contain only unicode objects, "\
                                    "object %r is not supported" % item)
            
            tname = 'combo'
        elif isinstance(v, unicode):
            tname = 'text'
        else:
            raise Exception("%s has non-supported value" % k)

        fields.append((unicode(k), tname, v))


    form = appuifw.Form(fields=fields, flags=appuifw.FFormEditModeOnly | \
                        appuifw.FFormDoubleSpaced)

    saved = [False]
    def save_hook(param):
        saved[0] = True
    form.save_hook = save_hook
    
    form.execute()

    # return true if user saved, false otherwise
    if not saved[0]:
        return False
    
    for label, tname, value in form:
        if tname == 'combo':
            value = (value[0], int(value[1]))

        cfg[str(label)] = value

    return True

def test():
    cfg = {'Well?': ([u"False", u"True"], 0),
           'Choose': ([u"A", u"B", u"C"], 1),
           'some number': 42,
           'any text': u"test"}    
    
    change = showconfig(cfg)
    print "did save?", change
    print cfg
    
if __name__ == "__main__":
    test()
