import thread
import sys
import os
import imp
import socket

import appuifw
import graphics
import e32
import iapconnect

import appswitch

if e32.in_emulator():
    BASEPATH = "p:\\dashboard\\python"
else:
    BASEPATH = os.path.dirname(sys.argv[0])

if BASEPATH not in sys.path:
    sys.path.append(BASEPATH)

import cfgform

# by sending a UDP packet to this port, this application will wake up
# and show itself (localhost only)
NOTIFY_UDP_PORT = 1031

X_SIZE = 176
Y_SIZE = 208

class DashBoardApp:
    def run(self):
        self.app_cfgfile = os.path.join(BASEPATH, "dashboard.cfg")
        self.app_cfg = eval(open(self.app_cfgfile).read())
        
        self.dashboard_img = graphics.Image.new((X_SIZE, Y_SIZE))
        self.dashboard_showing = False        
        
        appuifw.app.exit_key_handler = self.cmd_quit
        
        self.init_menu()
        
        self.load_plugins()
        self.init_plugins_listbox()
        
        self.start_listening_socket()

        self.external_launch = False
        
        self.ui_lock = e32.Ao_lock()
        self.ui_lock.wait()  # wait until exit is pressed

        appuifw.app.body = None

    def init_menu(self):
        #(u"placement", ((u"move up", lambda : move_selection('up')),
        #                (u"move down",lambda:move_selection('down')))),
        menu = [(u"configure", self.cmd_configure_selected_plugin),
                (u"show dashboard", self.cmd_show_dashboard),
                (u"list internet APs", self.cmd_show_iaps)]
        appuifw.app.menu = menu

    def cmd_quit(self):
        self.ui_lock.signal()

    def cmd_show_iaps(self):
        iap = iapconnect.iap_selection_dialog()
        appuifw.note(u"Enter number %d to weather->iap" % iap, 'info')
        
    def cmd_configure_selected_plugin(self):
        #sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        #sock.connect(('127.0.0.1', NOTIFY_UDP_PORT))
        #sock.send("foo")
        #return

        cfgfile = self.plugins[self.listbox.current()]['cfgfile']

        cfg = eval(open(cfgfile).read())
        
        if not cfg: # empty dict
            appuifw.note(u"no configuration needed for this plugin", 'info')
            return

        rc = cfgform.showconfig(cfg)
        if rc:
            open(cfgfile, "w").write(repr(cfg))
            self.plugins[self.listbox.current()]['config_cb'](cfg)
        
    def cmd_show_dashboard(self):
        appuifw.app.exit_key_handler = self.cmd_return_from_dashboard_mode
        appuifw.app.menu = []
        self.canvas = appuifw.Canvas(redraw_callback=self.handle_redraw)
        self.render_dashboard()
        appuifw.app.screen = "full"                
        appuifw.app.body = self.canvas
        
        if self.external_launch:
            if e32.in_emulator():
                name = u"Python"
            else:
                name = u"dashboard"
                
            appswitch.switch_to_fg(name)
            
    def cmd_return_from_dashboard_mode(self):
        if self.external_launch:
            appswitch.switch_to_fg(u"Phone")
            self.external_launch = False
        
        appuifw.app.exit_key_handler = self.cmd_quit
        appuifw.app.screen = "normal"
        appuifw.app.body = self.listbox
        del self.canvas
        self.init_menu()

    def render_dashboard(self):
        y_offset = 0
        for v in self.plugins:
            v['render_cb'](self.dashboard_img, (0, y_offset))
            y_offset += v['y_size']

        self.canvas.blit(self.dashboard_img)

    def handle_redraw(self, rect):
        self.canvas.blit(self.dashboard_img)
        
    def load_plugins(self):
        self.plugins = []
        
        PLUGINS_PATH = os.path.join(BASEPATH, "plugins")
        pluginlist = self.app_cfg['plugins'] # list of directory names
        
        for p in pluginlist:
            pluginpath = os.path.join(PLUGINS_PATH, p)
            pluginfile = os.path.join(pluginpath, "plugin.py")            
            if pluginpath not in sys.path:
                sys.path.append(pluginpath)
                
            plugin = imp.load_source("plugin", pluginfile, open(pluginfile))

            configfile = os.path.join(PLUGINS_PATH, p, "plugin.cfg")
            cfg = eval(open(configfile).read())
            
            render_cb, config_cb, y_size = plugin.init_plugin(cfg, X_SIZE)
            
            self.plugins.append({'name': p,
                                 'render_cb' : render_cb,
                                 'y_size' : y_size,
                                 'config_cb': config_cb,
                                 'cfgfile' : configfile})

    def init_plugins_listbox(self):
        pluginnames = [unicode(x['name']) for x in self.plugins]
        self.listbox = appuifw.Listbox(pluginnames, self.cmd_listbox_observe)
        appuifw.app.body = self.listbox
        
    def cmd_listbox_observe(self):
        self.cmd_configure_selected_plugin()
         
    def start_listening_socket(self):
        callgate = e32.ao_callgate(self.show_dashboard)
        thread.start_new_thread(_listen_and_notify, (callgate,
                                                     NOTIFY_UDP_PORT))

    def show_dashboard(self):
        # when 'full', dashboard is already shown
        if appuifw.app.screen != "full":
            self.external_launch = True
            self.cmd_show_dashboard()        

def _listen_and_notify(notify_callgate, udp_port):
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.bind(('0.0.0.0', udp_port))
    while 1:        
        data, addr = sock.recvfrom(1024)
        notify_callgate()


def run():
    DashBoardApp().run()

    if not e32.in_emulator():
        appuifw.app.set_exit()
