import time

import e32
import graphics
import calendar

import akntextutils

class CalendarPlugin:
    y_size = 300
    def __init__(self, config, x_size):
        self.config = config
        self.x_size = x_size
        self.cdb = calendar.open()

    def config_change(self, config):
        self.config = config
        
    def render_screen(self, bitmap, target_coords):
        img = graphics.Image.new((self.x_size, self.y_size))
        
        events = self.get_next_events(daylimit=100, eventlimit=5)

        y_offset = 0
        for event in events[:4]:
            y_offset += 14
            when = time.strftime("%a %d.%m %H:%M",
                                 time.localtime(event.start_time))
            line = "%s - %s" % (when, event.content)
            img.text((2, y_offset), unicode(line), font='dense')

        y_offset += max(0, 4-len(events)) * 14

        y_offset += 2
        img.line((0, y_offset, self.x_size, y_offset), width=1, outline=0x0)

        todos = self.get_todo_events()
        
        if self.config['ascending todo order'][1]:
            sorter = cmp
        else:
            sorter = lambda *x : not cmp(*x)
            
        todos.sort(lambda x, y: sorter(y.start_time, x.start_time))
        line = ", ".join([x.content.strip() for x in todos])
        line = line.replace("\u2029", " ")
        lines = akntextutils.wrap_text_to_array(line, 'dense', self.x_size)

        for line in lines[:3]:
            y_offset += 14
            img.text((2, y_offset), line, font='dense')

        bitmap.blit(img, target=target_coords)

    def get_todo_events(self):
        # THIS TO-DO API IS WICKED!!!!!!
        lst = self.cdb.todo_lists[self.cdb.todo_lists.default_list]
        return [self.cdb[lst[i]] for i in range(len(lst))]
    
    def get_next_events(self, daylimit, eventlimit):
        # get next events for next 'daylimit' days, but only
        # returning at maximum 'eventlimit' events

        # so, if you use daylimit=7, and eventlimit=3, it will return
        # at maximum 3 events which happen next 7 days.

        day = 24*60*60
        now = time.time()
        hlimit = now + daylimit*24*60*60

        events = []
        daycount = 0
        
        while daycount <= daylimit and len(events) < eventlimit:        
            entries = self.cdb.find_instances(now+(daycount*day), hlimit,
                                    appointments=1, events=1, anniversaries=1)
            if not entries:
                break

            for entry in entries:
                if entry not in events:
                    events.append(entry)

            daycount += 1

        return [self.cdb[x['id']] for x in events[:eventlimit]]

    
def init_plugin(config, x_size):
  cal_plugin = CalendarPlugin(config, x_size)
  return (cal_plugin.render_screen,
          cal_plugin.config_change,
          cal_plugin.y_size)


