import os
import sys

import e32
import graphics
import sysinfo

import exttimer

import yahooweather

# todo:
# - the iapconnect (used inside yahooparser) still bugs when closing down
#

class WeatherPlugin:
    y_size = 103
    if e32.in_emulator():
        ICON_PATH = "p:\\dashboard\\python\\pics\\%d.gif"
    else:
        ICON_PATH = os.path.join(os.path.dirname(sys.argv[0]), "pics\\%d.gif")
    
    def __init__(self, config, x_size):
        self.config = config
        self.x_size = x_size
        self.img = graphics.Image.new((self.x_size, self.y_size))
        e32.ao_sleep(0, self._update)

    def config_change(self, config):
        self.config = config
        
    def render_screen(self, bitmap, target_coords):
        bitmap.blit(self.img, target_coords)

    def _update(self):
        try:
            if e32.in_emulator() or sysinfo.battery() > 1:                
                self._do_update()            
        finally:            
            exttimer.schedule(self.config['interval'], self._update)

    def _do_update(self):
        #
        # IMPORTANT!
        #
        # Do create new bitmap for every rendering. This is because it
        # is possible that 'render_screen' can happen when this
        # function is executing. This means that the image we render
        # in 'render_screen' can be incomplete.
        #
        # This can happen because Image.open (and maybe other gfx funcs)
        # have nested active scheduler (I'm not 100% sure).
        #
        
        img = graphics.Image.new((self.x_size, self.y_size))
        
        info = yahooweather.get_weather_info(str(self.config['station']),
                                             self.config['iap'])
        fname = self.ICON_PATH % int(info['current'][1])
        cur_pic = graphics.Image.open(fname)
        img.blit(cur_pic, target=(0, 0), source=(0, 4))

        # forecast pics
        x_offset = 52
        for i in range(3):
            fname = self.ICON_PATH % int(info['forecast+%dd' % i][5])
            cur_pic = graphics.Image.open(fname)
            cur_pic = cur_pic.resize((41, 41))
            img.blit(cur_pic, target=(x_offset, 0), source=(0, 0))
            x_offset += 41

        y_offset = 57
        #self.img.text((0, offset), unicode(info['city']))        
        firstline = "%s: %s, %s%s" % (unicode(info['timestamp']), 
                                      unicode(info['current'][0]), 
                                      unicode(info['current'][2]),
                                      unicode(info['unit']))
        
        img.text((2, y_offset), firstline)
        
        for i in range(3):
            f = info['forecast+%dd' % i]
            fields = (f[0], f[4], f[2], f[3], info['unit'])
            txt = "%s: %s, %s - %s%s" % tuple(map(unicode, fields))
            y_offset += 14
            img.text((2, y_offset), txt, font='dense')

        y_offset += 2
        img.line((0, y_offset, self.x_size, y_offset), width=2, outline=0x0)
        self.img = img
    
def init_plugin(config, x_size):
  # here perform other setup code
  weather_plugin = WeatherPlugin(config, x_size)
  return (weather_plugin.render_screen,
          weather_plugin.config_change,
          weather_plugin.y_size)


