# fetch & parse data from Yahoo
# uses iapconnect for internet connection

from extralibs import BeautifulSoup
import time
import re

TESTING = 0
import e32
if e32.in_emulator():
    TESTING = 1
else:
    import iapconnect # only works in s60/python

def fetch_weather_data(default_iap, station):
    addr = u"xml.weather.yahoo.com" # NOTE: unicode!!
    path = "/forecastrss?p=%s&u=c" % station
    c = iapconnect.Connection()
    try:
        c.connect(addr, 80, default_iap)
        c.send("GET %s HTTP/1.0\n\n" % path)
        buf = []
        while 1:
            data = c.recv(4096)
            if not data:
                break
            buf.append(data)
            
        buf = ''.join(buf)
        return buf
    finally:
        c.stop()

if TESTING:
    def fetch_weather_data(default_iap, station):
        return open("c:\\temp\\forecastrss.xml").read()

def parse_weather_data(data):
    info = {}
    soup = BeautifulSoup.BeautifulSoup(data)

    # timestamp (extract HH:MM <am/pm>)
    ts = ' '.join(soup.first("pubdate").string.split()[4:6])
    info['timestamp'] = ts
    loc = soup.first("yweather:location")
    info['city'] = loc.attrs[0][1]
    info['country'] = loc.attrs[2][1]
    info['unit'] = soup.first("yweather:units").attrs[0][1]

    cur = soup.first("yweather:condition")
    # tuple (weather, image ref#, temp)
    info['current'] = (cur.attrs[0][1], cur.attrs[1][1], cur.attrs[2][1])

    #<yweather:forecast day="Mon" date="03 Oct 2005" low="8" high="15" text="Showers Late" code="45"/>
    fc = soup.first("yweather:forecast")
    # 3 day forecast (0, +1, +2 day)
    for i in range(3):
         info['forecast+%dd' % i] = [x[1] for x in fc.attrs]
         fc = fc.nextSibling

    return info

def get_weather_info(station, default_iap):
    # default_iap - internet access point id, symbian specific
    # station - yahoo's code for weather station, for example
    # helsinki, finland is "FIXX000"
    return parse_weather_data(fetch_weather_data(default_iap, station))
    
if __name__ == "__main__":
    print get_weather_info(0, 0)


