
#include <aputils.h> 
#include <apsettingshandlerui.h> 
#include <e32base.h>
#include <commdbconnpref.h>

#include "ConnManager.h"
#include "logger.h"

 

CConnManager::CConnManager():CActive(EPriorityStandard)
{
	CActiveScheduler::Add(this);
}

CConnManager::~CConnManager()
{	
	Cancel();
	if (iTimer)
		{
		iTimer->Cancel();
		}
	delete iTimer;


	iConnection.Close(); // this must be first, for some reason
	iSock.Close();	
	iResolver.Close();		
    iSocketServ.Close();		
	
	delete iWaitLoop;	
}

CConnManager*  CConnManager::NewL()
	{
	CConnManager* ctrl = new (ELeave) CConnManager;
	CleanupStack::PushL(ctrl);
	ctrl->ConstructL();
	CleanupStack::Pop(ctrl);
	return ctrl;
	}

void CConnManager::ConstructL()
{
	User::LeaveIfError(iSocketServ.Connect());
	User::LeaveIfError(iConnection.Open(iSocketServ));	
	iWaitLoop = new (ELeave) CActiveSchedulerWait;
	iTimer = CTimeOutTimer::NewL(EPriorityStandard, *this);
}

TInt CConnManager::SelectIAPL()
{
  TUint32 iap = 0;
  TUint32 selectedIap = 0;
  
  // Create Ap Settings Handler. See SDK Help for parameters.
  CApSettingsHandler* settingsHandler = CApSettingsHandler::NewLC(
    ETrue, 
    EApSettingsSelListIsPopUp, 
    EApSettingsSelMenuSelectOnly,
    KEApIspTypeAll, 
    EApBearerTypeAll, 
    KEApSortNameAscending);
    
  TUint32 originallyFocused(0);

  // Show the dialog
  settingsHandler->RunSettingsL(originallyFocused, selectedIap);

  // Original value 0 is not valid IAP id, so failed if 0 after RunSettingsL().
  if (selectedIap)
  {
    CCommsDatabase* db = CCommsDatabase::NewL();
    CleanupStack::PushL(db);
    CApUtils* apUtils = CApUtils::NewLC(*db);
    // Convert IAP id to CommsDb id
    iap = apUtils->IapIdFromWapIdL(selectedIap);   
    CleanupStack::PopAndDestroy(2);
  }
  else
  {
    iap = 0;
  }

  CleanupStack::PopAndDestroy();

  return (TInt) iap;
}


void CConnManager::ConnectL(const TDesC& aAddr, TInt aPort, TInt aAIP)
{		
	
	TCommDbConnPref prefs;
	prefs.SetDialogPreference(ECommDbDialogPrefDoNotPrompt);
	prefs.SetDirection(ECommDbConnectionDirectionOutgoing);
	prefs.SetIapId(aAIP);

	// Start an Outgoing Connection with overrides
	User::LeaveIfError(iConnection.Start(prefs));


	// check if valid ip. if not, resolve it
	if (iDestAddr.Input(aAddr) != KErrNone)
    {
		// Initiate DNS
		User::LeaveIfError(iResolver.Open(iSocketServ, KAfInet, KProtocolInetUdp));
		// DNS request for name resolution
		TNameEntry result;
		iResolver.GetByName(aAddr, result, iStatus); 
		SetActive();
		iWaitLoop->Start();
		User::LeaveIfError(iStatus.Int());
		iDestAddr.SetAddress(TInetAddr::Cast(result().iAddr).Address());
    }

	iDestAddr.SetPort(aPort);


	User::LeaveIfError(iSock.Open(iSocketServ, KAfInet, KSockStream, KProtocolInetTcp, iConnection));

	iSock.Connect(iDestAddr, iStatus);
	SetActive();
	iWaitLoop->Start();
	User::LeaveIfError(iStatus.Int());
}

void CConnManager::SoftCloseL()
{
	iSock.Close();	
}

void CConnManager::HardCloseL()
{
	// closes connection, even if there are active connections
	User::LeaveIfError(iConnection.Stop());	
}


void CConnManager::RunL()
{
	iWaitLoop->AsyncStop();
	iTimer->Cancel();
}

void CConnManager::DoCancel()
{
	iSock.CancelAll();
}

void CConnManager::RecvL(TDes8& aBuffer, TInt aTimeout)
{
	iSock.RecvOneOrMore(aBuffer, 0, iStatus, iXfrLength);
	SetActive();
	if (aTimeout)
	{
		iTimer->After(aTimeout);
	}
	iWaitLoop->Start();
	
	if (iStatus != KErrEof)
		{		
		// if not KErrNone or KErrEof, leave with error
		User::LeaveIfError(iStatus.Int());
		}	
}

void CConnManager::SendL(const TDesC8& aBuffer, TInt aTimeout)
{
	iSock.Write(aBuffer, iStatus);
	SetActive();
	if (aTimeout)
	{
		iTimer->After(aTimeout);
	}
	
	iWaitLoop->Start();
	User::LeaveIfError(iStatus.Int());
}


void CConnManager::TimerExpired()
	{
	Cancel();
	iStatus = KErrCancel;
	iWaitLoop->AsyncStop();
	}
