#include <e32std.h>
#include <e32base.h>

#include "Python.h"
#include "symbian_python_ext_util.h"

#include "ConnManager.h"

#define Connection_type_string "Connection.type_Connection"
#define type_Connection (*(PyTypeObject *)SPyGetGlobalString(Connection_type_string))


// type-definition
typedef struct {
  PyObject_HEAD;
	CConnManager* iConnManager;
} obj_Connection;


static PyObject* Connection_connect(obj_Connection* self, PyObject* args);
static PyObject* Connection_send(obj_Connection* self, PyObject* args);
static PyObject* Connection_recv(obj_Connection* self, PyObject* args);
static PyObject* Connection_softclose(obj_Connection* self, PyObject* args);
static PyObject* Connection_hardclose(obj_Connection* self, PyObject* args);


// ctor (factory-function)
static PyObject* Connection_Connection(PyObject* /*self*/, PyObject * /*args*/)
{
    obj_Connection *Connection = PyObject_New(obj_Connection, &type_Connection);

	if (!Connection)
		{
		return 0;
		}	
	
	Connection->iConnManager = 0;
	TRAPD(err,
		Connection->iConnManager = CConnManager::NewL();		
	);	

	if (err)
		{
		PyObject_Del(Connection);		
		return SPyErr_SetFromSymbianOSErr(err);
		}

     return (PyObject*)Connection;
}

// dtor
static void dealloc_Connection(obj_Connection* Connection)
{   
	delete Connection->iConnManager;
	Connection->iConnManager = 0;
    PyObject_Del(Connection);    
}
 


static const PyMethodDef Connection_methods[] =
  {
    {"connect", (PyCFunction)Connection_connect, METH_VARARGS},
	{"send", (PyCFunction)Connection_send, METH_VARARGS},    
	{"recv", (PyCFunction)Connection_recv, METH_VARARGS},    
	{"close", (PyCFunction)Connection_softclose, METH_NOARGS},    
	{"stop", (PyCFunction)Connection_hardclose, METH_NOARGS},    
    {NULL, NULL} /* sentinel */
  };

static PyObject *getattr_Connection(PyObject *self, char *name)
{
  return Py_FindMethod(const_cast<PyMethodDef*>(&Connection_methods[0]), self, name);
}

static const PyTypeObject type_template_Connection = {
/*******************************************************/
    PyObject_HEAD_INIT(0)    /* initialize to 0 to ensure Win32 portability */
    0,                 /*ob_size*/
    "Connection.Connection",            /*tp_name*/
    sizeof(obj_Connection), /*tp_basicsize*/
    0,                 /*tp_itemsize*/
    /* methods */
    (destructor)dealloc_Connection, /*tp_dealloc*/
	 0, /*tp_print*/
    (getattrfunc)getattr_Connection, /*tp_getattr*/

    /* implied by ISO C: all zeros thereafter */
}; 


// Connection methods

static PyObject* Connection_connect(obj_Connection* self, PyObject* args)
{
	TInt iap;
	TInt port;
	char* b = NULL;	
	TInt l = 0;

	if (!PyArg_ParseTuple(args, "u#ii", &b, &l, &port, &iap))
	{
		return 0;
	}
	
	if (iap == 0)
		{
		PyErr_SetString(PyExc_Exception, "iap id cannot be 0");
		return 0;
		}

	TInt err;
	Py_BEGIN_ALLOW_THREADS;
	TRAP(err,
		TPtrC addr((TUint16*)b, l);		
        self->iConnManager->ConnectL(addr, port, iap);
		);
	Py_END_ALLOW_THREADS;

	if (err)
	{
		return SPyErr_SetFromSymbianOSErr(err); 
	}

	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* Connection_send(obj_Connection* self, PyObject* args)
{
	char* b = NULL;	
	TInt l = 0;
	TInt timeout = 0;

	if (!PyArg_ParseTuple(args, "s#|i", &b, &l, &timeout))
	{
		return 0;
	}
	
	TInt err;
	Py_BEGIN_ALLOW_THREADS;
	TRAP(err,
		TPtrC8 buf((TUint8*)b, l);		
        self->iConnManager->SendL(buf, timeout);
		);
	Py_END_ALLOW_THREADS;

	if (err)
	{
		return SPyErr_SetFromSymbianOSErr(err); 
	}

	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* Connection_recv(obj_Connection* self, PyObject* args)
{	
	TInt l = 0;
	TInt timeout = 0;

	if (!PyArg_ParseTuple(args, "i|i", &l, &timeout))
	{
		return 0;
	}
	
	HBufC8* buf;
	TInt err;
	bool alloc_ok = EFalse;

	Py_BEGIN_ALLOW_THREADS;			
	TRAP(err,
		buf = HBufC8::NewLC(l);
		TPtr8 tmp(buf->Des());
		alloc_ok = ETrue;
        self->iConnManager->RecvL(tmp, timeout);
		CleanupStack::Pop();
		);
	Py_END_ALLOW_THREADS;

	if (err)
	{	
		if (alloc_ok)
			{
			delete buf;
			}		
		return SPyErr_SetFromSymbianOSErr(err); 
	}
	
	PyObject* rv = Py_BuildValue("s#", buf->Ptr(), buf->Length());
	delete buf;
	return rv;
}

static PyObject* Connection_hardclose(obj_Connection* self, PyObject* /*args*/)
{
	TInt err;
	Py_BEGIN_ALLOW_THREADS;
	TRAP(err,
        self->iConnManager->HardCloseL();
		);
	Py_END_ALLOW_THREADS;

	if (err)
	{
		return SPyErr_SetFromSymbianOSErr(err); 
	}
	
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* Connection_softclose(obj_Connection* self, PyObject* /*args*/)
{
	TInt err;
	Py_BEGIN_ALLOW_THREADS;
	TRAP(err,
        self->iConnManager->SoftCloseL();
		);
	Py_END_ALLOW_THREADS;
		
	if (err)
	{
		return SPyErr_SetFromSymbianOSErr(err); 
	}
	
	Py_INCREF(Py_None);
	return Py_None;
}


static PyObject* SelectIAP(PyObject* /*self*/, PyObject * /*args*/)
{
	TInt iap = 0;
	TInt err;

	Py_BEGIN_ALLOW_THREADS;
	TRAP(err,
		iap = CConnManager::SelectIAPL();
	);
	Py_END_ALLOW_THREADS;

	if (err)
	{	
		return SPyErr_SetFromSymbianOSErr(err);
	}
  
  return PyInt_FromLong(iap);
}



static const PyMethodDef iapconnect_methods[] =
{
	{"Connection", (PyCFunction)Connection_Connection, METH_NOARGS},		
	{"iap_selection_dialog", (PyCFunction)SelectIAP, METH_NOARGS},
	{0, 0} /* sentinel */
};

DL_EXPORT(void) init_iapconnect()
{
	// work done here:
	// 1. init module
    // 2. create type object for our Connection type


	PyObject* module = Py_InitModule("iapconnect", (PyMethodDef*) iapconnect_methods);
	if (!module)
		return;

	PyTypeObject *ConnectionTypeObject = PyObject_New(PyTypeObject, &PyType_Type);
	if (!ConnectionTypeObject)
		return;

    *ConnectionTypeObject = type_template_Connection;

	TInt err = SPyAddGlobalString(Connection_type_string, (PyObject *)ConnectionTypeObject);
    if (0 != err) // 0 is success
		{
		PyObject_Del(ConnectionTypeObject);
		PyErr_SetString(PyExc_Exception, "SPyAddGlobalString failed");
		return;
		}
    
    // notice that the this uses the macro defined in the beginning of file
    type_Connection.ob_type = &PyType_Type; 
}

GLDEF_C TInt E32Dll(TDllReason)
{
	return KErrNone;
}


