/* Copyright (c) 2003, Nokia. All rights reserved */

#ifndef __HELLOWORLDBASIC_DOCUMENT_H__
#define __HELLOWORLDBASIC_DOCUMENT_H__


#include <akndoc.h>

class CHelloWorldBasicAppUi;
class CEikApplication;


/*! 
  @class CHelloWorldBasicDocument
  
  @discussion An instance of class CHelloWorldBasicDocument is the Document part of the AVKON
  application framework for the HelloWorldBasic example application
  */
class CHelloWorldBasicDocument : public CAknDocument
    {
public:

/*!
  @function NewL
  
  @discussion Construct a CHelloWorldBasicDocument for the AVKON application aApp 
  using two phase construction, and return a pointer to the created object
  @param aApp The application creating this document
  @result A pointer to the created instance of CHelloWorldBasicDocument
  */
    static CHelloWorldBasicDocument* NewL(CEikApplication& aApp);

/*!
  @function NewLC
  
  @discussion Construct a CHelloWorldBasicDocument for the AVKON application aApp 
  using two phase construction, and return a pointer to the created object
  @param aApp The application creating this document
  @result A pointer to the created instance of CHelloWorldBasicDocument
  */
    static CHelloWorldBasicDocument* NewLC(CEikApplication& aApp);

/*!
  @function ~CHelloWorldBasicDocument
  
  @discussion Destroy the object
  */
    ~CHelloWorldBasicDocument();

public: // from CAknDocument
/*!
  @function CreateAppUiL 
  
  @discussion Create a CHelloWorldBasicAppUi object and return a pointer to it
  @result A pointer to the created instance of the AppUi created
  */
    CEikAppUi* CreateAppUiL();

private:

/*!
  @function ConstructL
  
  @discussion Perform the second phase construction of a CHelloWorldBasicDocument object
  */
    void ConstructL();

/*!
  @function CHelloWorldBasicDocument
  
  @discussion Perform the first phase of two phase construction 
  @param aApp The application creating this document
  */
    CHelloWorldBasicDocument(CEikApplication& aApp);

    };


#endif // __HELLOWORLDBASIC_DOCUMENT_H__
